/**
 * RC_COPY for AHCC (c) 2011 by Simon Sunnyboy / Paradize
 */

#include <tos.h>
#include <stdlib.h>
#include <stdint.h>
#include "rccopy.h"


/**
 * RC_COPY equivalent in C utilizing LINEA function 7
 * This function blits a pixel rectangle from a source screen to a destination screen.
 * Both screens should be in ST-LOW bitplane format.
 *
 * @param src pointer to/address of source screen
 * @param src_x upperleft x coordinate of source block within source screen
 * @param src_y upperleft y coordinate of source block within source screen
 * @param w width of block in pixels
 * @param h width of block in pixels
 * @param dest pointer to/address of destination screen
 * @param dest_x upperleft x coordinate on destination screen
 * @param dest_y upperleft y coordinate on destination screen
 * @param op  blit operation mode (consult a LINEA documentation)
 */
void RC_COPY ( void *src,int16_t src_x,int16_t src_y, int16_t w, int16_t h, void *dest,int16_t dest_x, int16_t dest_y, int8_t op )
{
	/* empty structure for blit - prepared for ST-LOW on the stack */
	/* we place this on the stack because LINEA alters the parameters while working
	 * that renders the structure unusable for the next blit
	 */
	BITBLT param =
	{
		0,     /* Breite des Blocks in Pixeln  */
		0,     /* Höhe des Blocks in Pixeln    */
		4,     /* Anzahl der Farbplanes        */
		15,     /* Vordergrundfarbe             */
		0,     /* Hintergrundfarbe             */
		{ 0,0,0,0},     /* Verknüpfung (für jede Plane) */

		0,     /* X-Quellraster                */
		0,     /* Y-Quellraster                */
		NULL,     /* Adresse des Quellrasters     */
		8,     /* Offset zum nächsten Wort     */
		160,     /* Breite des Quellrasters      */
		2,     /* Offset zur nächsten Plane    */

		0,     /* X-Zielraster                 */
		0,     /* Y-Zielraster                 */
		NULL,     /* Adresse des Zielrasters      */
		8,     /* Offset zum nächsten Wort     */
		160,     /* Breite des Quellrasters      */
		2,     /* Offset zur nächsten Plane    */

		NULL,     /* 16-Bit-Masken zum Undieren   */
		0,     /* Breite der Maske in Bytes    */
		0,     /* Offset zur nächsten Plane    */
		0,     /* Höhe der Maske in Zeilen     */

		{0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,
		 0,0,0,0,0,0,0,0,}
	};
	/* fill in the user details for the requested blit: */
	param.b_wd = w; /* set width and height of raster block in pixels */
	param.b_ht = h;
	param.op_tab[0]=op; /* set blit operation for each plane */
	param.op_tab[1]=op; /* 1 = AND  7 = OR   3 = 1:1 */
	param.op_tab[2]=op;
	param.op_tab[3]=op;
	param.s_xmin = src_x; /* upperleft coordinate on source screen*/
	param.s_ymin = src_y;
	param.s_form = src;   /* pointer to start of source screen */
	param.d_xmin = dest_x; /* upperleft coordinate on destination screen*/
	param.d_ymin = dest_y;
	param.d_form = dest;   /* pointer to start of destination screen */
	LineA_BITBLT ( &param ); /* blit the block with LINEA */
	return;
}

