/**
 * RC_COPY for AHCC (c) 2011 by Simon Sunnyboy / Paradize
 */

#ifndef __RCOPY_H
#define __RCOPY_H

#include <stdint.h>

/**
 * font struct needed for LineA
 */
typedef struct font_hdr
{
   int16_t     font_id;        /* Zeichensatz-Nummer                    */
   int16_t     point;          /* Größe in Punkten                      */
   int8_t      name[32];       /* Name des Zeichensatzes                */
   uint16_t    first_ade;      /* erstes Zeichen im Zeichensatz         */
   uint16_t    last_ade;       /* letztes Zeichen im Zeichensatz        */
   uint16_t    top;            /* Abstand: Topline     <-> Baseline     */
   uint16_t    ascent;         /* Abstand: Ascentline  <-> Baseline     */
   uint16_t    half;           /* Abstand: Halfline    <-> Baseline     */
   uint16_t    descent;        /* Abstand: Descentline <-> Baseline     */
   uint16_t    bottom;         /* Abstand: Bottomline  <-> Baseline     */
   uint16_t    max_char_width; /* größte Zeichenbreite                  */
   uint16_t    max_cell_width; /* größte Zeichenzellenbreite            */
   uint16_t    left_offset;    /* linker Offset für Kursivschrift       */
   uint16_t    right_offset;   /* rechter Offset für Kursivschrift      */
   uint16_t    thicken;        /* Verbreiterungsfaktor für Fettschrift  */
   uint16_t    ul_size;        /* Dicke der Unterstreichung             */
   uint16_t    lighten;        /* Maske für helle Schrift (0x5555)      */
   uint16_t    skew;           /* Maske für Kursivschrift (0x5555)      */
   uint16_t    flags;          /* verschiedene Flags:                   */
                               /* gesetzt falls System-Font             */
                               /* Bit-1: gesetzt falls Horizontal       */
                               /*        Offset-Table benutzt wird      */
                               /* Bit-2: gesetzt falls Motorola-Format  */
                               /* Bit-3: gesetzt falls nicht proport.   */
   uint8_t    *hor_table;      /* Zeiger auf Horizontal-Offset-Table    */
   uint16_t    *off_table;     /* Zeiger auf Character-Offset-Table     */
   uint16_t    *dat_table;     /* Zeiger auf Fontimage                  */
   uint16_t    form_width;     /* Breite des Zeichensatz-Image          */
   uint16_t    form_height;    /* Höhe des Zeichensatz-Image            */
   struct font_hdr *next_font;        /* Zeiger auf nächsten Font-Header       */
} FONT_HDR;

/**
 * LINEA parameter block
 */
typedef struct
{
  int16_t  v_planes,               /*   0: # Bitplanes (1, 2 od. 4)     */
           v_lin_wr,               /*   2: # Bytes/Scanline             */
           *contrl,
           *intin,
           *ptsin,                 /*  12: Koordinaten-Eingabe          */
           *intout,
           *ptsout,                /*  20: Koordinaten-Ausgabe          */
           fg_bp_1,                /*  24: Plane 0                      */
           fg_bp_2,                /*  26: Plane 1                      */
           fg_bp_3,                /*  28: Plane 2                      */
           fg_bp_4,                /*  30: Plane 3                      */
           lstlin;                 /*  32: letzten Pixel einer Linie    */
                                   /*      zeichen (1) oder nicht (0)   */
  uint16_t ln_mask;                /*  34: Linienmuster                 */
  int16_t  wrt_mode,               /*  36: Schreib-Modus                */
           x1, y1, x2, y2;         /*  38: Koordinaten                  */
  void     *patptr;                /*  46: Füllmuster                   */
  uint16_t patmsk;                 /*  50: .. dazugehörige Maske        */
  int16_t  multifill,              /*  52: Füllmuster für Planes        */
           clip,                   /*  54: Flag für Clipping            */
           xmn_clip, ymn_clip,
           xmx_clip, ymx_clip,     /*  60: Clipping Rechteck            */
                                   /*      Rest für text_blt:           */
           xacc_dda,               /*  64: vor Textausgaben auf 0x8000  */
                                   /*      setzen                       */
           dda_inc,                /*  66: Vergrößerungsfaktor          */
           t_sclsts,               /*  68: Vergrößerungsrichtung        */
           mono_status,            /*  70: Proportionalschrift          */
           sourcex, sourcey,       /*  72: Koordinaten im Font          */
           destx, desty,           /*  76: Bildschirmkoordinaten        */
           delx, dely;             /*  80: Breite und Höhe des Zeichen  */
  FONT_HDR *fbase;                 /*  84: Start der Font-Daten         */
  int16_t  fwidth,                 /*  88: Breite des Fontimage         */
           style;                  /*  90: Schreibstil                  */
  uint16_t litemask,               /*  92: Maske für Light              */
           skewmask;               /*  94: Maske für Kursiv             */
  int16_t  weight,                 /*  96: Breite bei Bold              */
           r_off,                  /*  98: Kursiv-Offset rechts         */
           l_off,                  /* 100: Kursiv-Offset links          */
           scale,                  /* 102: Vergrößerung ja/nein         */
           chup,                   /* 104: Rotationswinkel *10          */
           text_fg;                /* 106: Textfarbe                    */
  void     *scrtchp;               /* 108: Buffer                       */
  int16_t  scrpt2,                 /* 112: Index in Buffer              */
           text_bg,                /* 114: unbenutzt                    */
           copy_tran,              /* 116: --                           */
           (*fill_abort)( void );  /* 118: Testet Seed Fill             */
} LINEA;


/**
 * Main BITBLT access struct
 */
typedef struct
{
  int16_t  b_wd,         /* Breite des Blocks in Pixeln  */
           b_ht,         /* Höhe des Blocks in Pixeln    */
           plane_ct,     /* Anzahl der Farbplanes        */
           fg_col,       /* Vordergrundfarbe             */
           bg_col;       /* Hintergrundfarbe             */
  int8_t   op_tab[4];    /* Verknüpfung (für jede Plane) */
  int16_t  s_xmin,       /* X-Quellraster                */
           s_ymin;       /* Y-Quellraster                */
  void    *s_form;       /* Adresse des Quellrasters     */
  int16_t  s_nxwd,       /* Offset zum nächsten Wort in der gleichen Plane    */
           s_nxln,       /* Breite des Quellrasters  pro Scanline in Bytes    */
           s_nxpl,       /* Offset zur nächsten Plane    */
           d_xmin,       /* X-Zielraster                 */
           d_ymin;       /* Y-Zielraster                 */
  void    *d_form;       /* Adresse des Zielrasters      */
  int16_t  d_nxwd,       /* Offset zum nächsten Wort     */
           d_nxln,       /* Breite des Quellrasters      */
           d_nxpl;       /* Offset zur nächsten Plane    */
  void    *p_addr;       /* 16-Bit-Masken zum Undieren   */
  int16_t  p_nxln,       /* Breite der Maske in Bytes    */
           p_nxpl,       /* Offset zur nächsten Plane    */
           p_mask;       /* Höhe der Maske in Zeilen     */
  int8_t   filler[24];   /* Interner Buffer              */
} BITBLT;

LINEA* LineA_Init(void);
void LineA_BITBLT(BITBLT *param);

void RC_COPY(void *src,int16_t src_x,int16_t src_y, int16_t w, int16_t h, void *dest,int16_t dest_x, int16_t dest_y, int8_t op);

#endif

